from pytplot import options

def mms_fpi_set_metadata(probe, data_rate, datatype, level, suffix=''):
    """
    This function updates the metadata for FPI data products
    
    Parameters:
        probe : str or list of str
            probe or list of probes, valid values for MMS probes are ['1','2','3','4']. 

        data_rate : str or list of str
            instrument data rates for FPI include 'brst' and 'fast'. The
            default is 'fast'.

        level : str
            indicates level of data processing. the default if no level is specified is 'l2'

        suffix: str
            The tplot variable names will be given this suffix.  By default, 
            no suffix is added.

    """
    if not isinstance(probe, list): probe = [probe]
    if not isinstance(data_rate, list): data_rate = [data_rate]
    if not isinstance(datatype, list): datatype = [datatype]
    if not isinstance(level, list): level = [level]

    probe = [str(p) for p in probe]
    
    for this_probe in probe:
        for this_dr in data_rate:
            for this_lvl in level:
                for this_dtype in datatype:
                    if this_dtype == 'des-moms':
                        options('mms'+this_probe+'_des_energyspectr_par_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES (eV)')
                        options('mms'+this_probe+'_des_energyspectr_anti_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES (eV)')
                        options('mms'+this_probe+'_des_energyspectr_perp_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES (eV)')
                        options('mms'+this_probe+'_des_energyspectr_omni_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES (eV)')

                        options('mms'+this_probe+'_des_energyspectr_par_'+this_dr+suffix, 'ylog', True)
                        options('mms'+this_probe+'_des_energyspectr_par_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_des_energyspectr_par_'+this_dr+suffix, 'Colormap', 'jet')
                        
                        options('mms'+this_probe+'_des_energyspectr_anti_'+this_dr+suffix, 'ylog', True)
                        options('mms'+this_probe+'_des_energyspectr_anti_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_des_energyspectr_anti_'+this_dr+suffix, 'Colormap', 'jet')
                        
                        options('mms'+this_probe+'_des_energyspectr_perp_'+this_dr+suffix, 'ylog', True)
                        options('mms'+this_probe+'_des_energyspectr_perp_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_des_energyspectr_perp_'+this_dr+suffix, 'Colormap', 'jet')
                        
                        options('mms'+this_probe+'_des_energyspectr_omni_'+this_dr+suffix, 'ylog', True)
                        options('mms'+this_probe+'_des_energyspectr_omni_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_des_energyspectr_omni_'+this_dr+suffix, 'Colormap', 'jet')
                        
                        options('mms'+this_probe+'_des_pitchangdist_lowen_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_des_pitchangdist_lowen_'+this_dr+suffix, 'Colormap', 'jet')

                        options('mms'+this_probe+'_des_pitchangdist_miden_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_des_pitchangdist_miden_'+this_dr+suffix, 'Colormap', 'jet')

                        options('mms'+this_probe+'_des_pitchangdist_highen_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_des_pitchangdist_highen_'+this_dr+suffix, 'Colormap', 'jet')

                        options('mms'+this_probe+'_des_pitchangdist_lowen_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES (deg)')
                        options('mms'+this_probe+'_des_pitchangdist_miden_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES (deg)')
                        options('mms'+this_probe+'_des_pitchangdist_highen_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES (deg)')

                        options('mms'+this_probe+'_des_energyspectr_par_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')
                        options('mms'+this_probe+'_des_energyspectr_anti_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')
                        options('mms'+this_probe+'_des_energyspectr_perp_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')
                        options('mms'+this_probe+'_des_energyspectr_omni_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')

                        options('mms'+this_probe+'_des_pitchangdist_lowen_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')
                        options('mms'+this_probe+'_des_pitchangdist_miden_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')
                        options('mms'+this_probe+'_des_pitchangdist_highen_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')


                        options('mms'+this_probe+'_des_bulkv_dbcs_'+this_dr+suffix, 'color', ['b', 'g', 'r'])
                        options('mms'+this_probe+'_des_bulkv_gse_'+this_dr+suffix, 'color', ['b', 'g', 'r'])

                        options('mms'+this_probe+'_des_bulkv_gse_'+this_dr+suffix, 'legend_names', ['Vx GSE', 'Vy GSE', 'Vz GSE'])
                        options('mms'+this_probe+'_des_bulkv_dbcs_'+this_dr+suffix, 'legend_names', ['Vx DBCS', 'Vy DBCS', 'Vz DBCS'])
                        options('mms'+this_probe+'_des_bulkv_gse_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES velocity (km/s)')
                        options('mms'+this_probe+'_des_bulkv_dbcs_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES velocity (km/s)')
                        options('mms'+this_probe+'_des_numberdensity_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DES density (cm^-3)')
                    elif this_dtype == 'dis-moms':
                        options('mms'+this_probe+'_dis_energyspectr_omni_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DIS (eV)')
                        options('mms'+this_probe+'_dis_energyspectr_omni_'+this_dr+suffix, 'ylog', True)
                        options('mms'+this_probe+'_dis_energyspectr_omni_'+this_dr+suffix, 'zlog', True)
                        options('mms'+this_probe+'_dis_energyspectr_omni_'+this_dr+suffix, 'Colormap', 'jet')
                        options('mms'+this_probe+'_dis_energyspectr_omni_'+this_dr+suffix, 'ztitle', '[keV/(cm^2 s sr keV)]')

                        options('mms'+this_probe+'_dis_bulkv_dbcs_'+this_dr+suffix, 'color', ['b', 'g', 'r'])
                        options('mms'+this_probe+'_dis_bulkv_gse_'+this_dr+suffix, 'color', ['b', 'g', 'r'])

                        options('mms'+this_probe+'_dis_bulkv_gse_'+this_dr+suffix, 'legend_names', ['Vx GSE', 'Vy GSE', 'Vz GSE'])

                        options('mms'+this_probe+'_dis_bulkv_dbcs_'+this_dr+suffix, 'legend_names', ['Vx DBCS', 'Vy DBCS', 'Vz DBCS'])

                        options('mms'+this_probe+'_dis_bulkv_gse_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DIS velocity (km/s)')
                        options('mms'+this_probe+'_dis_bulkv_dbcs_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DIS velocity (km/s)')

                        options('mms'+this_probe+'_dis_numberdensity_'+this_dr+suffix, 'ytitle', 'MMS'+this_probe+' DIS density (cm^-3)')







# -*- coding: utf-8 -*-
"""
File:
    ex_basic.py

Desrciption:
    Basic example to verify that the installation works correctly.
    Downloads THEMIS data and plots it.

"""

import pyspedas
import pytplot


def ex_basic():
    # Delete any existing pytplot variables
    pytplot.del_data()
    # Download THEMIS state data for 2015-12-31
    pyspedas.load_data('themis', ['2015-12-31 00:00:00',
                       '2016-01-01 12:00:00'], ['tha'], 'state', 'l1')
    # Get data into python variables
    time, data = pytplot.get_data("tha_pos")
    # Store a new pytplot variable
    pytplot.store_data("tha_position", data={'x': time, 'y': data})
    # Define the y-axis limits
    pytplot.ylim('tha_pos', -23000.0, 81000.0)
    pytplot.ylim('tha_position', -23000.0, 81000.0)
    pytplot.ylim('tha_vel', -8.0, 12.0)
    # Plot position and velocity using the pyqtgraph library (default)
    pytplot.tplot(["tha_pos", "tha_position", "tha_vel"])
    # Plot position and velocity using the bokeh library
    pytplot.tplot(["tha_pos", "tha_position", "tha_vel"], bokeh=True)

# Run the example code
# ex_basic()



from .load_data import load_data
from .version import version
from .prefs import get_spedas_prefs
from .helpers import download_files, url_exists, find_latest_url_version
from .dates import validate_date, get_date_list, get_dates
from .themis.themis_load import themis_filename, themis_load
from .themis.gmag_load import gmag_load, gmag_filename, check_gmag, check_greenland, gmag_groups, gmag_list, get_group, query_gmags
from .themis.themis_helpers import get_probes, get_instruments
from .omni.omni_load import omni_filename, omni_load
from .examples.basic.ex_basic import ex_basic
from .examples.basic.ex_analysis import ex_analysis
from .examples.basic.ex_gmag import ex_gmag
from .examples.basic.ex_spectra import ex_spectra
from .examples.basic.ex_line_spectra import ex_line_spectra
from .examples.basic.ex_create import ex_create
from .examples.basic.ex_omni import ex_omni
from .utilities.data_exists import data_exists
from .utilities.tnames import tnames
from .utilities.time_string import time_string
from .utilities.time_double import time_float, time_double
from .utilities.tcopy import tcopy
from .utilities.tplot2ascii import tplot2ascii
from .analysis.tclip import tclip
from .analysis.tdeflag import tdeflag
from .analysis.tinterpol import tinterpol
from .analysis.subtract_average import subtract_average
from .analysis.subtract_median import subtract_median
from .analysis.time_clip import time_clip
from .spdtplot.cdf_to_tplot import cdf_to_tplot
from .spdtplot.tplot_names import tplot_names

from .mms import mms_load_mec, mms_load_fgm, mms_load_scm, mms_load_edi, mms_load_edp, mms_load_eis, mms_load_feeps, mms_load_hpca, mms_load_fpi, mms_load_aspoc, mms_load_dsp



# -*- coding: utf-8 -*-
"""
File:
    time_string.py

Desrciption:
    Transforms datetimes from decimal to string.

Parameters:
    float_time: float/list of floats
    If None, then uses Now.

Returns:
    str/list of str
    A time string ('YYYY-MM-DD HH:MM:SS.NNN') .

Examples:
    time_string()
    time_string(None, '%Y-%m-%d')
    time_string([1498780799.1234, 1512087321.1234])

Notes:
    Test with https://www.epochconverter.com/
"""

from datetime import datetime


def time_string_one(float_time=None, fmt=None):

    if fmt is None:
        fmt = '%Y-%m-%d %H:%M:%S.%f'

    if float_time is None:
        str_time = datetime.now().strftime(fmt)
    else:
        str_time = datetime.utcfromtimestamp(float_time).strftime(fmt)

    return str_time


def time_string(float_time=None, fmt=None):

    if float_time is None:
        return time_string_one(None, fmt)
    else:
        if isinstance(float_time, (int, float)):
            return time_string_one(float_time, fmt)
        else:
            time_list = list()
            for t in float_time:
                time_list.append(time_string_one(t, fmt))
            return time_list



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import requests
import logging
import numpy as np
from ..spdtplot.cdf_to_tplot import cdf_to_tplot
from ..analysis.time_clip import time_clip as tclip
from pyspedas import time_double, time_string
from dateutil.parser import parse
from datetime import timedelta, datetime
from shutil import copyfileobj, copy
from tempfile import NamedTemporaryFile
from .mms_config import CONFIG
from .mms_get_local_files import mms_get_local_files
from .mms_files_in_interval import mms_files_in_interval
from .mms_login_lasp import mms_login_lasp

logging.captureWarnings(True)
logging.basicConfig(format='%(asctime)s: %(message)s', datefmt='%d-%b-%y %H:%M:%S', level=logging.INFO)

def mms_load_data(trange=['2015-10-16', '2015-10-17'], probe='1', data_rate='srvy', level='l2', 
    instrument='fgm', datatype='', prefix='', suffix='', get_support_data=False, time_clip=False, 
    no_update=False, center_measurement=False, available=False):
    """
    This function loads MMS data into pyTplot variables
    """

    if not isinstance(probe, list): probe = [probe]
    if not isinstance(data_rate, list): data_rate = [data_rate]
    if not isinstance(level, list): level = [level]
    if not isinstance(datatype, list): datatype = [datatype]
    
    probe = [str(p) for p in probe]

    # allows the user to pass in trange as list of datetime objects
    if type(trange[0]) == datetime and type(trange[1]) == datetime:
        trange = [time_string(trange[0].timestamp()), time_string(trange[1].timestamp())]
        
    start_date = parse(trange[0]).strftime('%Y-%m-%d') # need to request full day, then parse out later
    end_date = parse(time_string(time_double(trange[1])-0.1)).strftime('%Y-%m-%d-%H-%M-%S') # -1 second to avoid getting data for the next day

    download_only = CONFIG['download_only']

    no_download = False
    if no_update or CONFIG['no_download']: no_download = True

    user = None
    if not no_download:
        sdc_session, user = mms_login_lasp()

    out_files = []
    available_files = []

    for prb in probe:
        for drate in data_rate:
            for lvl in level:
                for dtype in datatype:
                    if user is None:
                        url = 'https://lasp.colorado.edu/mms/sdc/public/files/api/v1/file_info/science?start_date=' + start_date + '&end_date=' + end_date + '&sc_id=mms' + prb + '&instrument_id=' + instrument + '&data_rate_mode=' + drate + '&data_level=' + lvl
                    else:
                        url = 'https://lasp.colorado.edu/mms/sdc/sitl/files/api/v1/file_info/science?start_date=' + start_date + '&end_date=' + end_date + '&sc_id=mms' + prb + '&instrument_id=' + instrument + '&data_rate_mode=' + drate + '&data_level=' + lvl
                    
                    if dtype != '':
                        url = url + '&descriptor=' + dtype

                    if CONFIG['debug_mode']: logging.info('Fetching: ' + url)

                    if no_download == False:
                        # query list of available files
                        try:
                            http_json = sdc_session.get(url, verify=True).json()

                            if CONFIG['debug_mode']: logging.info('Filtering the results down to your trange')

                            files_in_interval = mms_files_in_interval(http_json['files'], trange)

                            if available:
                                for file in files_in_interval:
                                    logging.info(file['file_name'] + ' (' + str(np.round(file['file_size']/(1024.*1024), decimals=1)) + ' MB)')
                                    available_files.append(file['file_name'])
                                continue

                            for file in files_in_interval:
                                file_date = parse(file['timetag'])
                                if dtype == '':
                                    out_dir = os.sep.join([CONFIG['local_data_dir'], 'mms'+prb, instrument, drate, lvl, file_date.strftime('%Y'), file_date.strftime('%m')])
                                else:
                                    out_dir = os.sep.join([CONFIG['local_data_dir'], 'mms'+prb, instrument, drate, lvl, dtype, file_date.strftime('%Y'), file_date.strftime('%m')])

                                if drate.lower() == 'brst':
                                    out_dir = os.sep.join([out_dir, file_date.strftime('%d')])

                                out_file = os.sep.join([out_dir, file['file_name']])

                                if CONFIG['debug_mode']: logging.info('File: ' + file['file_name'] + ' / ' + file['timetag'])

                                #if os.path.exists(out_file) and str(os.stat(out_file).st_size) == str(file['file_size']):
                                if os.path.exists(out_file):
                                    if not download_only: logging.info('Loading ' + out_file)
                                    out_files.append(out_file)
                                    continue

                                if user is None:
                                    download_url = 'https://lasp.colorado.edu/mms/sdc/public/files/api/v1/download/science?file=' + file['file_name']
                                else:
                                    download_url = 'https://lasp.colorado.edu/mms/sdc/sitl/files/api/v1/download/science?file=' + file['file_name']

                                logging.info('Downloading ' + file['file_name'] + ' to ' + out_dir)

                                fsrc = sdc_session.get(download_url, stream=True, verify=True)
                                ftmp = NamedTemporaryFile(delete=False)
                                copyfileobj(fsrc.raw, ftmp)

                                if not os.path.exists(out_dir):
                                    os.makedirs(out_dir)

                                # if the download was successful, copy to data directory
                                copy(ftmp.name, out_file)
                                out_files.append(out_file)
                                ftmp.close()
                        except requests.exceptions.ConnectionError:
                            # No/bad internet connection; try loading the files locally
                            logging.error('No internet connection!')

                    
                    if out_files == []:
                        if not download_only: logging.info('Searching for local files...')
                        out_files = mms_get_local_files(prb, instrument, drate, lvl, dtype, trange)

    if not no_download:
        sdc_session.close()

    if available:
        return available_files

    if not download_only:
        out_files = sorted(out_files)

        new_variables = cdf_to_tplot(out_files, merge=True, get_support_data=get_support_data, prefix=prefix, suffix=suffix, center_measurement=center_measurement)

        if new_variables == []:
            logging.warning('No data loaded.')
            return

        logging.info('Loaded variables:')
        for new_var in new_variables:
            print(new_var)

            if time_clip:
                tclip(new_var, trange[0], trange[1], suffix='')

        return new_variables









import pytplot

def mms_feeps_split_integral_ch(units_type, species, probe, suffix='', data_rate='srvy', level='l2', sensor_eyes=None):
    if sensor_eyes is None:
        print('Error: sensor_eyes not defined')
        return

    top_sensors = sensor_eyes['top']
    bot_sensors = sensor_eyes['bottom']

    for sensor in top_sensors:
        top_name = 'mms'+str(probe)+'_epd_feeps_'+data_rate+'_'+level+'_'+species+'_top_'+units_type+'_sensorid_'+str(sensor)

        time, data = pytplot.get_data(top_name+suffix)
        energies = pytplot.data_quants[top_name+suffix].spec_bins.values

        top_name_out = top_name+'_clean'+suffix
        pytplot.store_data(top_name_out, data={'x': time, 'y': data[:, :-1], 'v': energies[:, :-1].reshape(energies[:, :-1].size)})

        pytplot.store_data(top_name+'_500keV_int'+suffix, data={'x': time, 'y': data[:, -1]})

        pytplot.del_data(top_name)

    if level == 'sitl': # SITL only has top sensors
        return

    for sensor in bot_sensors:
        bot_name = 'mms'+str(probe)+'_epd_feeps_'+data_rate+'_'+level+'_'+species+'_bottom_'+units_type+'_sensorid_'+str(sensor)

        time, data = pytplot.get_data(bot_name+suffix)
        energies = pytplot.data_quants[bot_name+suffix].spec_bins.values

        bot_name_out = bot_name+'_clean'+suffix
        pytplot.store_data(bot_name_out, data={'x': time, 'y': data[:, :-1], 'v': energies[:, :-1].reshape(energies[:, :-1].size)})

        pytplot.store_data(bot_name+'_500keV_int'+suffix, data={'x': time, 'y': data[:, -1]})

        pytplot.del_data(bot_name)



# Copyright 2018 Regents of the University of Colorado. All Rights Reserved.
# Released under the MIT license.
# This software was developed at the University of Colorado's Laboratory for Atmospheric and Space Physics.
# Verify current version before use at: https://github.com/MAVENSDC/PyTplot

import cdflib
import re
import numpy as np
import pandas as pd
from pytplot.store_data import store_data
from pytplot.tplot import tplot
from pytplot.options import options
# from pytplot.get_data import get_data
from pytplot import data_quants


def cdf_to_tplot(filenames, varformat=None, get_support_data=False,
                 prefix='', suffix='', plot=False, merge=False, 
                 center_measurement=False):
    """
    This function will automatically create tplot variables from CDF files.

    .. note::
        Variables must have an attribute named "VAR_TYPE".  If the attribute entry
        is "data" (or "support_data"), then they will be added as tplot variables.
        Additionally, data variables should have attributes named "DEPEND_TIME" or
        "DEPEND_0" that describes which variable is x axis.  If the data is 2D,
        then an attribute "DEPEND_1" must describe which variable contains the
        secondary axis.


    Parameters:
        filenames : str/list of str
            The file names and full paths of CDF files.
        varformat : str
            The file variable formats to load into tplot.  Wildcard character
            "*" is accepted.  By default, all variables are loaded in.
        get_support_data: bool
            Data with an attribute "VAR_TYPE" with a value of "support_data"
            will be loaded into tplot.  By default, only loads in data with a
            "VAR_TYPE" attribute of "data".
        prefix: str
            The tplot variable names will be given this prefix.  By default,
            no prefix is added.
        suffix: str
            The tplot variable names will be given this suffix.  By default,
            no suffix is added.
        plot: bool
            The data is plotted immediately after being generated.  All tplot
            variables generated from this function will be on the same plot.
        merge: bool
            If True, then data from different cdf files will be merged into
            a single pytplot variable.

    Returns:
        List of tplot variables created.
    """

    stored_variables = []
    epoch_cache = {}
    global data_quants

    if isinstance(filenames, str):
        filenames = [filenames]
    elif isinstance(filenames, list):
        filenames = filenames
    else:
        print("Invalid filenames input.")
        return stored_variables

    var_type = ['data']
    if varformat == None:
        varformat = ".*"
    if get_support_data:
        var_type.append('support_data')

    try:
        varformat = varformat.replace("*", ".*")
        var_regex = re.compile(varformat)
    except:
        print("Error reading the varformat.")
        return

    for filename in filenames:
        cdf_file = cdflib.CDF(filename)
        cdf_info = cdf_file.cdf_info()
        all_cdf_variables = cdf_info['rVariables'] + cdf_info['zVariables']

        # Find the data variables
        for var in all_cdf_variables:
            if not re.match(var_regex, var):
                continue
            var_atts = cdf_file.varattsget(var)

            if 'VAR_TYPE' not in var_atts:
                continue

            if var_atts['VAR_TYPE'] in var_type:
                var_properties = cdf_file.varinq(var)
                if "DEPEND_TIME" in var_atts:
                    x_axis_var = var_atts["DEPEND_TIME"]
                elif "DEPEND_0" in var_atts:
                    x_axis_var = var_atts["DEPEND_0"]
                else:
                    if var_atts['VAR_TYPE'].lower() == 'data':
                        print("Cannot find x axis.")
                        print("No attribute named DEPEND_TIME or DEPEND_0 in variable " + var)
                    continue
                data_type_description = cdf_file.varinq(x_axis_var)['Data_Type_Description']

                # Find data name and if it is already in stored variables
                var_name = prefix + var + suffix
                to_merge = False
                if (var_name in data_quants.keys()) and (merge == True):
                    prev_data_quant = data_quants[var_name].data
                    to_merge = True

                if epoch_cache.get(filename+x_axis_var) is None:
                    delta_plus_var = 0.0
                    delta_minus_var = 0.0
                    delta_time = 0.0

                    xdata = cdf_file.varget(x_axis_var)
                    epoch_var_atts = cdf_file.varattsget(x_axis_var)

                    # check for DELTA_PLUS_VAR/DELTA_MINUS_VAR attributes
                    if center_measurement:
                        if 'DELTA_PLUS_VAR' in epoch_var_atts:
                            delta_plus_var = cdf_file.varget(epoch_var_atts['DELTA_PLUS_VAR'])
                            delta_plus_var_att = cdf_file.varattsget(epoch_var_atts['DELTA_PLUS_VAR'])

                            # check if a conversion to seconds is required
                            if 'SI_CONVERSION' in delta_plus_var_att:
                                si_conv = delta_plus_var_att['SI_CONVERSION']
                                delta_plus_var = delta_plus_var.astype(float)*np.float(si_conv.split('>')[0])
                            elif 'SI_CONV' in delta_plus_var_att:
                                si_conv = delta_plus_var_att['SI_CONV']
                                delta_plus_var = delta_plus_var.astype(float)*np.float(si_conv.split('>')[0])

                        if 'DELTA_MINUS_VAR' in epoch_var_atts:
                            delta_minus_var = cdf_file.varget(epoch_var_atts['DELTA_MINUS_VAR'])
                            delta_minus_var_att = cdf_file.varattsget(epoch_var_atts['DELTA_MINUS_VAR'])

                            # check if a conversion to seconds is required
                            if 'SI_CONVERSION' in delta_minus_var_att:
                                si_conv = delta_minus_var_att['SI_CONVERSION']
                                delta_minus_var = delta_minus_var.astype(float)*np.float(si_conv.split('>')[0])
                            elif 'SI_CONV' in delta_minus_var_att:
                                si_conv = delta_minus_var_att['SI_CONV']
                                delta_minus_var = delta_minus_var.astype(float)*np.float(si_conv.split('>')[0])

                        # sometimes these are specified as arrays
                        if isinstance(delta_plus_var, np.ndarray) and isinstance(delta_minus_var, np.ndarray):
                            delta_time = (delta_plus_var-delta_minus_var)/2.0
                        else: # and sometimes constants
                            if delta_plus_var != 0.0 or delta_minus_var != 0.0:
                                delta_time = (delta_plus_var-delta_minus_var)/2.0

                    if ('CDF_TIME' in data_type_description) or ('CDF_EPOCH' in data_type_description):
                        xdata = cdflib.cdfepoch.unixtime(xdata)
                        epoch_cache[filename+x_axis_var] = np.array(xdata)+delta_time
                else:
                    xdata = epoch_cache[filename+x_axis_var]

                try:
                    ydata = cdf_file.varget(var)
                except:
                    continue

                if ydata is None:
                    continue
                if "FILLVAL" in var_atts:
                    if (var_properties['Data_Type_Description'] == 'CDF_FLOAT' or
                            var_properties['Data_Type_Description'] == 'CDF_REAL4' or
                            var_properties['Data_Type_Description'] == 'CDF_DOUBLE' or
                            var_properties['Data_Type_Description'] == 'CDF_REAL8'):

                        if ydata[ydata == var_atts["FILLVAL"]].size != 0:
                            ydata[ydata == var_atts["FILLVAL"]] = np.nan

                tplot_data = {'x': xdata, 'y': ydata}

                depend_1 = None
                depend_2 = None
                depend_3 = None
                if "DEPEND_1" in var_atts:
                    if var_atts["DEPEND_1"] in all_cdf_variables:
                        depend_1 = cdf_file.varget(var_atts["DEPEND_1"])
                if "DEPEND_2" in var_atts:
                    if var_atts["DEPEND_2"] in all_cdf_variables:
                        depend_2 = cdf_file.varget(var_atts["DEPEND_2"])
                if "DEPEND_3" in var_atts:
                    if var_atts["DEPEND_3"] in all_cdf_variables:
                        depend_3 = cdf_file.varget(var_atts["DEPEND_3"])

                if (depend_1 is not None) and (depend_2 is not None) and (depend_3 is not None):
                    tplot_data['v1'] = depend_1
                    tplot_data['v2'] = depend_2
                    tplot_data['v3'] = depend_3
                elif (depend_1 is not None) and (depend_2 is not None):
                    tplot_data['v1'] = depend_1
                    tplot_data['v2'] = depend_2
                elif depend_1 is not None:
                    tplot_data['v'] = depend_1
                elif depend_2 is not None:
                    tplot_data['v'] = depend_2

                store_data(var_name, data=tplot_data)
                if var_name not in stored_variables:
                    stored_variables.append(var_name)

                display_type = var_atts.get("DISPLAY_TYPE", "time_series")
                scale_type = var_atts.get("SCALE_TYP", "linear")
                if display_type == "spectrogram":
                    options(var_name, 'spec', 1)
                if scale_type == 'log':
                    options(var_name, 'ylog', 1)

                if to_merge:
                    cur_data_quant = data_quants[var_name].data
                    merged_data = [prev_data_quant, cur_data_quant]
                    data_quants[var_name].data = pd.concat(merged_data)

        cdf_file.close() if hasattr(cdf_file, "close") else None

    if plot:
        tplot(stored_variables)

    return stored_variables


# script for pySPEDAS tutorial at the 2019 Winter Community Workshop
# agenda:
# 1. installation; requirements
# 2. loading MMS data
# 3. plotting MMS data
# 4. working with the data vales, timestamps and metadata
# 5. analysis tools

###############################################################

# 2. loading MMS data

from pyspedas.mms import mms_load_fgm, mms_load_eis

mms_load_fgm(trange=['2015-10-16', '2015-10-17'])

# the load routines accept all of the standard keywords
mms_load_eis(probe='4', data_rate='brst', datatype=['extof', 'phxtof'], trange=['2015-10-16/13:05', '2015-10-16/13:10'])

# list the current tplot variables

from pytplot import tplot_names

tplot_names()

from pyspedas.mms import mms_load_fpi, mms_load_eis

###############################################################
# 3. plotting MMS data

from pytplot import tplot

tplot('mms1_fgm_b_gsm_srvy_l2')

tplot(['mms4_epd_eis_brst_phxtof_proton_flux_omni', 'mms4_epd_eis_brst_extof_proton_flux_omni'])

from pyspedas.mms import mms_load_fpi

mms_load_fpi(probe='4', data_rate='brst', datatype='des-moms', trange=['2015-10-16/13:00', '2015-10-16/13:10'])

tplot(['mms4_des_energyspectr_omni_brst', 'mms4_des_pitchangdist_miden_brst', 'mms4_des_bulkv_gse_brst', 'mms4_des_numberdensity_brst'])


###############################################################
# 4. working with the data vales, timestamps and metadata

from pytplot import get_data, store_data

# extract the data from a tplot variable
times, data = get_data('mms1_fgm_b_gsm_srvy_l2')

# e.g.,
# >>> times[0]
# 1444953613.330852
# >>> data[0]
# array([ 8580.49 ,  7339.21 ,  6250.034, 12905.493], dtype=float32)

# convert the unix time to a string

from pyspedas.utilities.time_string import time_string

print(time_string(1444953613.330852))

from pyspedas.utilities.time_double import time_double

print(time_double('2015-10-16 00:00:13.330852'))

# create new tplot variables

store_data('b_gsm_vec', data={'x': times, 'y': data[:,0:3]}) # B-field vector

store_data('b_gsm_mag', data={'x': times, 'y': data[:,3]}) # B-field magnitude

tplot(['b_gsm_mag', 'b_gsm_vec'])

# modify variable metadata

from pytplot import options

options('b_gsm_mag', 'ytitle', 'B-field magnitude')
options('b_gsm_vec', 'ytitle', 'B-field vector')
options('b_gsm_vec', 'color', ['b', 'g', 'r'])
options('b_gsm_vec', 'legend_names', ['Bx', 'By', 'Bz'])

tplot(['b_gsm_mag', 'b_gsm_vec'])

# save the data to ASCII files
from pyspedas.utilities.tplot2ascii import tplot2ascii
tplot2ascii(['b_gsm_mag', 'b_gsm_vec'])

###############################################################
# 5. analysis tools

# clip to a certain time
from pyspedas.analysis.time_clip import time_clip

time_clip('b_gsm_vec', '2015-10-16/10:00', '2015-10-16/14:00')

options('b_gsm_vec-tclip', 'color', ['r', 'g', 'b'])
options('b_gsm_vec', 'legend_names', ['Bx', 'By', 'Bz'])
tplot('b_gsm_vec-tclip')

# clip the data
from pyspedas.analysis.tclip import tclip

tclip('b_gsm_vec-tclip', -20, 20)

# subtract the average
from pyspedas.analysis.subtract_average import subtract_average

subtract_average('mms1_fgm_b_gsm_srvy_l2')

tplot('mms1_fgm_b_gsm_srvy_l2-d')

